##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Builder interfaces

$Id: interfaces.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import zope.interface
import zope.schema
from zope.container.constraints import contains, containers
from zope.container.interfaces import IOrderedContainer, IContained
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

troveClassifiers = open(os.path.join(os.path.dirname(__file__),
                                     'trove-classifiers.txt')).read().split('\n')
troveClassiferVocabulary = SimpleVocabulary(map(SimpleTerm,
                                                troveClassifiers))

class FileExistsException(Exception):
    """Exception to throw if a file that will be written already exists."""

    filename = None

    def __init__(self, filename, message=u''):
        super(FileExistsException, self).__init__(message)
        self.filename = filename


# --- Basic Builder Components ------------------------------------------------

class IBaseBuilder(IContained):
    """A builder constructs code from a programmatic desciption."""

    name = zope.schema.TextLine(
        title=u'Name',
        description=u'The name of the builder instance.')

    def update():
        """Update the builder before rendering.

        The purpose of this method is to prepare the builder and dependents
        for rendering.
        """

class IBuilderContainer(IBaseBuilder, IOrderedContainer):
    """A container for other builders."""
    contains(IBaseBuilder)

    def add(builder):
        """Add a builder to the container.

        The name is usually computed from the name of the builder.

        The name of the builder in the component is returned.
        """

    def remove(builder):
        """Remove the builder from the container."""


class IProjectGetter(zope.interface.Interface):
    """Gets the project the builser belongs to."""

    def getProject():
        """Return the project instance."""


class IContentBuilder(IBaseBuilder):
    """Build a piece of content/code within a file."""

    def render():
        """Render the code and return the result as a string."""


class IFilesystemBuilder(IBaseBuilder):
    """Builds and writes a filesystem-level component."""

    def write(target):
        """Write the new file(s) into the target directory."""


class IFileBuilder(IFilesystemBuilder):
    """Bulder to construct a single file with all its content."""

    filename = zope.schema.ASCIILine(
        title=u'File Name',
        description=u'The name of the file to be created.')


class IDirectoryBuilder(IFilesystemBuilder, IBuilderContainer):
    """Builder to construct a directory and all its content."""
    contains(IFileBuilder, '.IDirectoryBuilder')

    dirname = zope.schema.ASCIILine(
        title=u'Directory Name',
        description=u'The name of the directory to be created.')


# --- Python Code Builders ----------------------------------------------------

class IPythonPathRoot(zope.interface.Interface):
    """Marker interface flagging the root directory."""


class IPythonPathGetter(zope.interface.Interface):
    """Gets a full Python path of the current builder."""

    def getPythonPath():
        """Return full Python path as a string."""


class IModuleBuilderGetter(zope.interface.Interface):
    """Gets the next module builder."""

    def getModuleBuilder():
        """Return the next module builder."""


class IPackageBuilder(IDirectoryBuilder, IPythonPathGetter):
    """Builds a full Python package or sub-package."""

    packageName = zope.schema.ASCIILine(
        title=u'Package Name',
        description=u'The name of the package to be created.',
        required=True)

    initTemplate = zope.schema.Bytes(
        title=u'Init Template',
        description=u'The content template for the `__init__.py` file.',
        required=True)


class IModuleBuilder(IFileBuilder, IBuilderContainer, IPythonPathGetter):
    """Builds a Python module."""

    moduleName = zope.schema.ASCIILine(
        title=u'Module Name',
        description=u'The name of the module to be created.')

    imports = zope.schema.List(
        title=u'Import List',
        description=u'The list of objects to import.')


# --- Content Builders --------------------------------------------------------

class IFunctionBuilder(IContentBuilder):
    """A builder to construct a function."""

    args = zope.schema.Tuple(
        title=u'Args',
        description=u'List of function arguments.',
        required=False)

    kwargs = zope.schema.Dict(
        title=u'Kwargs',
        description=u'<apping of function keyword arguments.',
        required=False)

    docstring = zope.schema.ASCII(
        title=u'Interface Docstring',
        description=u'The docstring of the function.')

    indent = zope.schema.Int(
        title=u'Indentation Level',
        description=u'The number of columns to indent the function.',
        default=0)

    code = zope.schema.ASCII(
        title=u'Code',
        description=u'The source code of the function.')


class IFieldBuilder(IContentBuilder):
    """A builder to create a field for an interface builder."""

    fieldName = zope.schema.ASCIILine(
        title=u'Field Name',
        description=u'The name of the field.',
        required=True)

    type = zope.schema.ASCIILine(
        title=u'Type',
        description=u'The full Python path of the type of the field.',
        required=True)

    attributes = zope.schema.Dict(
        title=u'Attributes',
        key_type = zope.schema.ASCIILine(),
        value_type = zope.schema.Field(),
        description=u'The attributes of the field.')

    indent = zope.schema.Int(
        title=u'Indentation Level',
        description=u'The number of columns to indent the field.',
        default=0)


class IInterfaceBuilder(IPythonPathGetter, IContentBuilder):
    """A builder to construct interfaces and schemas."""

    docstring = zope.schema.ASCII(
        title=u'Interface Docstring',
        description=u'The docstring of the itnerface.')

    bases = zope.schema.List(
        title=u'Bases',
        description=u'A list of base interfaces.')


class IClassFromInterfaceBuilder(IContentBuilder):

    #className = zope.schema.TextLine(
    #    title=u'Class Name',
    #    description=u'The name of the generated class.')

    interface = zope.schema.Field(
        title=u'Interface',
        description=u'The interface to generate the class from.')

    docstring = zope.schema.ASCII(
        title=u'Interface Docstring',
        description=u'The docstring of the interface.')

    bases = zope.schema.List(
        title=u'Bases',
        description=u'A list of base interfaces (full Python path).',
        value_type=zope.schema.DottedName(title=u'Full Python Path'))

    implementations = zope.schema.Dict(
        title=u'Implementations',
        description=u'A collection of method implementations.',
        key_type=zope.schema.ASCIILine(),
        value_type=zope.schema.ASCII())


# --- Setup Builders ---------------------------------------------------------

class ISetupBuilder(IFileBuilder):

    version = zope.schema.TextLine(
        title=u"Version",
        default=u"0.1.0",
        required=False)

    license = zope.schema.TextLine(
        title=u"License",
        default=u"GPLv3",
        required=False)

    author = zope.schema.TextLine(
        title=u"Author",
        default=u"",
        required=False)

    author_email = zope.schema.TextLine(
        title=u"Author Email",
        default=u"",
        required=False)

    description = zope.schema.TextLine(
        title=u"Description",
        default=u"",
        required=False)

    keywords = zope.schema.List(
        title=u"Keywords",
        value_type=zope.schema.TextLine(),
        required=False)

    namespace_packages = zope.schema.List(
        title=u'Namespace Packages',
        value_type=zope.schema.TextLine(),
        required=False)

    url = zope.schema.TextLine(
        title=u'URL',
        required=False)

    classifiers = zope.schema.List(
        title=u"Trove Classifiers",
        value_type=zope.schema.Choice(
            vocabulary=troveClassiferVocabulary),
        required=False)

    install_requires = zope.schema.List(
        title=u"Install Requires",
        value_type=zope.schema.ASCIILine(),
        required=False)

    extras_require = zope.schema.Dict(
        title=u"Extras Require",
        key_type=zope.schema.TextLine(),
        value_type=zope.schema.List(value_type=zope.schema.TextLine()))

    entry_points = zope.schema.Dict(
        title=u"Extras Require",
        key_type=zope.schema.TextLine(),
        value_type=zope.schema.List(value_type=zope.schema.TextLine()))

    def addExtrasRequires(name, requirements):
        """Add the requirements to the named extra.

        Multiple calls should not override each other.
        """

    def removeExtrasRequires(name):
        """Remove the requirements from the named extra."""

    def addEntryPoints(name, entries):
        """Add entry points for the given name.

        Multiple calls should not override each other.
        """

    def removeEntryPoints(name):
        """Remove the entry points from the named section."""


# --- Buildout Config Builders ------------------------------------------------

class IPartBuilder(IFileBuilder):

    values = zope.schema.List(
        title=u"Values",
        description=(u"A list of values set for this part."),
        required=True)

    autoBuild = zope.schema.Bool(
        title=u"Auto-Build",
        description=(u"A flag, when set, causes the part to be built "
                     u"automatically"),
        default=True,
        required=True)

    def addValue(key, value):
        """Add a value to the part."""

    def removeValue(key):
        """Remove a value by key."""


class IBuildoutConfigBuilder(IBuilderContainer, IFileBuilder):
    """An object generating the  `buildout.cfg` file."""

    extends = zope.schema.List(
        title=u"Extends",
        description=(u"A list of buildout configurations used to extend the "
                     u"configuration file."),
        value_type=zope.schema.TextLine(),
        required=True)

    names = zope.schema.List(
        title=u"Names",
        description=u"A list of part names that are automatically built.",
        value_type=zope.schema.TextLine(),
        required=True)


# --- ZCML Builders -----------------------------------------------------------

class IZCMLDirectiveBuilder(IBuilderContainer, IContentBuilder):
    """A ZCML Directive Builder."""

    namespace = zope.schema.URI(
        title=u'Namespace',
        description=u'Namespace URI of the directive.',
        required=False)

    #name = zope.schema.TextLine(
    #    title=u'Name',
    #    description=u'The directive name.',
    #    required=True)

    attributes = zope.schema.Dict(
        title=u'Attributes',
        description=u'A collection of attributes of the ZCML directive.',
        required=False)

    indent = zope.schema.Int(
        title=u'Indentation Level',
        description=u'The number of columns to indent the directive.',
        default=0)

    def getZCMLBuilder():
        """Get the ZCML File Builder."""


class IZCMLFileBuilder(IZCMLDirectiveBuilder, IFileBuilder):
    """A ZCML File Builder."""

    i18n_domain = zope.schema.TextLine(
        title=u'I18n Domain',
        description=u'The I18n domain to use for localization.',
        required=False)

    namespaces = zope.schema.List(
        title=u'Namespaces',
        description=u'A list of namespaces used in the file.',
        required=True)


# --- Form Builders -----------------------------------------------------------

class IFormBuilder(IContentBuilder):
    """Generic form builder."""

    interface = zope.schema.ASCIILine(
        title=u'Interface',
        description=u'Full path to the interface.')

    fields = zope.schema.Tuple(
        title=u'Fields',
        description=u'A list of fields to display.',
        value_type=zope.schema.ASCIILine())


class IAddFormBuilder(IFormBuilder):
    """Add form builder."""

    label = zope.schema.TextLine(
        title=u'Label',
        description=u'A label of the form.')

    factory = zope.schema.ASCIILine(
        title=u'Factory',
        description=u'Full path to the factory.')

    next = zope.schema.ASCII(
        title=u'Next URL',
        description=u'The relative URL to go to after adding the object.')


class IEditFormBuilder(IFormBuilder):
    """Edit form builder."""

    label = zope.schema.TextLine(
        title=u'Label',
        description=u'A label of the form.')


class IDisplayFormBuilder(IFormBuilder):
    """Display form builder."""

    template = zope.schema.Text(
        title=u'Template',
        description=u'A template for displaying the data.',
        required=False)


# --- Project Builders --------------------------------------------------------

class IProjectBuilder(IDirectoryBuilder):
    """A Python Project Builder."""

    projectName = zope.schema.ASCIILine(
        title=u"Project Name")

    commentHeader = zope.schema.Text(
        title=u"Comment Header",
        required=False)

    def write(target, force=False):
        """Write the new project in the target directory.

        If `force` is set, then any existing project in the target path is
        ignored, and the new one is wrtten over it.
        """

class IBuildoutProjectBuilder(IProjectBuilder):
    """A Buildout-based Python Project Builder."""

    package = zope.schema.Object(
        title=u"Project Code Package",
        schema=IPackageBuilder,
        readonly=True)

    setup = zope.schema.Object(
        title=u"Project Setup",
        schema=ISetupBuilder,
        readonly=True)

    buildout = zope.schema.Object(
        title=u"Project Buildout",
        schema=IFileBuilder,
        readonly=True)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Helpers

$Id: testing.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import logging
import os
import shutil
import subprocess
import sys
import tempfile
from z3c.builder.core import base

class TestingHandler(logging.StreamHandler):

    def __init__(self):
        logging.StreamHandler.__init__(self, sys.stdout)

    def flush(self):
        self.stream = sys.stdout
        logging.StreamHandler.flush(self)

    def emit(self, record):
        self.stream = sys.stdout
        logging.StreamHandler.emit(self, record)


def formatDirListing(path):
    result = ''
    listing = os.listdir(path)
    files = filter(lambda x: os.path.isfile(os.path.join(path, x)), listing)
    files.sort()
    dirs = filter(lambda x: os.path.isdir(os.path.join(path, x)), listing)
    dirs.sort()
    result += '\n'.join(files)
    if dirs and files:
        result += '\n'
    for name in dirs:
        subPath = os.path.join(path, name)
        result += '%s/\n' % name
        result += '  '+formatDirListing(subPath).replace('\n','\n  ')
    return result

def ls(path, *args):
    path = os.path.join(path, *args)
    print formatDirListing(path)

def more(*args):
    path = os.path.join(*args)
    print open(path, 'r').read()

def clear(path):
    shutil.rmtree(path)
    os.mkdir(path)

def cmd(args, cwd):
    outputFilename = tempfile.mktemp()
    output = open(outputFilename, 'w')
    cmd = subprocess.Popen(args, cwd=cwd, stdout=output, stderr=output)
    status = cmd.wait()
    print 'Exit Status: ' + str(status)
    output.close()
    return open(outputFilename, 'r').read()

def buildSetUp(test):
    fn = tempfile.mktemp()
    test.globs.update(
        {'buildPath': fn,
         'cmd': cmd,
         'more': more,
         'ls':ls,
         'clear': clear})
    os.mkdir(fn)

def buildTearDown(test):
    shutil.rmtree(test.globs['buildPath'])

def loggerSetUp(test, level=logging.DEBUG, stream=None):
    test._oldLevel = base.logger.level
    if stream is None:
        test._handler = TestingHandler()
    else:
        test._handler = logging.StreamHandler(stream)
    base.logger.setLevel(level)
    base.logger.addHandler(test._handler)


def loggerTearDown(test):
    base.logger.setLevel(test._oldLevel)
    base.logger.removeHandler(test._handler)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Base Builder Components

$Id: base.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import logging
import os
import uuid
import zope.interface
from zope.schema.fieldproperty import FieldProperty
from zope.container import contained, ordered
from z3c.builder.core import interfaces

formatter = logging.Formatter('%(levelname)s - %(message)s')
logger = logging.getLogger('info')

def getTemplatePath(fn):
    return os.path.join(os.path.dirname(__file__), 'file-templates', fn)

def getUUID():
    return str(uuid.uuid4())

class ProjectGetter(object):
    zope.interface.implements(interfaces.IProjectGetter)

    def getProject(self):
        project = self
        while not interfaces.IProjectBuilder.providedBy(project):
            project = project.__parent__
            if project is None:
                raise ValueError(
                    'No project builder was found and the root node of '
                    'the project tree was reached.')
        return project


class BaseBuilder(contained.Contained):
    zope.interface.implements(interfaces.IBaseBuilder)

    name = FieldProperty(interfaces.IBaseBuilder['name'])

    def __init__(self, name):
        self.name = name

    def update(self):
        """See interfaces.IBaseBuilder"""
        logger.debug("Updating %s" %self)

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.name)


class BuilderContainer(BaseBuilder, ordered.OrderedContainer):
    zope.interface.implements(interfaces.IBuilderContainer)

    def __init__(self, name):
        super(BuilderContainer, self).__init__(name)
        ordered.OrderedContainer.__init__(self)

    def update(self):
        """See interfaces.IBaseBuilder"""
        super(BuilderContainer, self).update()
        for subBuilder in self.values():
            subBuilder.update()

    def add(self, builder):
        """See interfaces.IBuilderContainer"""
        self[builder.name] = builder
        return builder.name

    def remove(self, builder):
        """See interfaces.IBuilderContainer"""
        del self[builder.__name__]


class ContentBuilder(BaseBuilder, ProjectGetter):
    zope.interface.implements(interfaces.IContentBuilder)

    def render(self):
        """See interfaces.IContentBuilder"""
        raise NotImplementedError(
            "The `render()` method is not implemented by %s." %self)


class FilesystemBuilder(BaseBuilder, ProjectGetter):
    zope.interface.implements(interfaces.IFilesystemBuilder)

    def write(self, target):
        """See interfaces.IFilesystemBuilder"""
        raise NotImplementedError(
            "The `write(target)` method is not implemented.")


class DirectoryBuilder(FilesystemBuilder, BuilderContainer):
    zope.interface.implements(interfaces.IDirectoryBuilder)

    dirname = FieldProperty(interfaces.IDirectoryBuilder['dirname'])

    def __init__(self, name, dirname=None):
        super(DirectoryBuilder, self).__init__(name)
        # Use the builder name as the default directory name.
        if dirname is None:
            dirname = str(name)
        self.dirname = dirname

    def write(self, target):
        """See interfaces.IFilesystemBuilder"""
        dirpath = os.path.join(target, self.dirname)
        logger.info("Creating directory %s" % dirpath)
        os.mkdir(dirpath)
        for subBuilder in self.values():
            subBuilder.write(dirpath)


class FileBuilder(FilesystemBuilder):
    zope.interface.implements(interfaces.IFileBuilder)

    filename = FieldProperty(interfaces.IFileBuilder['filename'])

    def __init__(self, name, filename=None):
        super(FileBuilder, self).__init__(name)
        if filename is None:
            filename = str(name)
        self.filename = filename

    def render(self):
        """See interfaces.IContentBuilder"""
        raise NotImplementedError("The `render()` method is not implemented.")

    def write(self, target):
        """See interfaces.IFilesystemBuilder"""
        filepath = os.path.join(target, self.filename)
        logger.info("Creating file %s" % filepath)
        file = open(filepath, 'w')
        file.write(self.render())
        file.close()


class SimpleFileBuilder(FileBuilder):

    def __init__(self, name, template, filename=None):
        super(SimpleFileBuilder, self).__init__(name, filename)
        self.template = template

    def render(self):
        """See interfaces.IContentBuilder"""
        return open(self.template, 'r').read()



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Representation of the setup.py file.

$Id: setup.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import types
import pprint
import zope.interface
from rwproperty import getproperty
from zope.schema.fieldproperty import FieldProperty
from zope.container.sample import SampleContainer
from zope.container.contained import Contained
from zope.location.location import locate

from z3c.builder.core import base, buildout, interfaces

class SetupBuilder(base.FileBuilder):
    zope.interface.implements(interfaces.ISetupBuilder)

    version = FieldProperty(interfaces.ISetupBuilder['version'])
    license = FieldProperty(interfaces.ISetupBuilder['license'])
    author = FieldProperty(interfaces.ISetupBuilder['author'])
    author_email = FieldProperty(interfaces.ISetupBuilder['author_email'])
    description = FieldProperty(interfaces.ISetupBuilder['description'])
    keywords = FieldProperty(interfaces.ISetupBuilder['keywords'])
    url = FieldProperty(interfaces.ISetupBuilder['url'])
    classifiers = FieldProperty(
        interfaces.ISetupBuilder['classifiers'])
    namespace_packages = FieldProperty(
        interfaces.ISetupBuilder['namespace_packages'])
    install_requires = FieldProperty(
        interfaces.ISetupBuilder['install_requires'])
    extras_requires = FieldProperty(
        interfaces.ISetupBuilder['extras_require'])
    entry_points = FieldProperty(
        interfaces.ISetupBuilder['entry_points'])

    def __init__(self):
        super(SetupBuilder, self).__init__(u'setup.py')
        self.keywords = []
        self.namespace_packages = []
        self.install_requires = []
        self.classifiers = []
        self.extras_requires = {}
        self.entry_points = {}

    def addExtrasRequires(self, name, requirements):
        """See interfaces.ISetupBuilder"""
        self.extras_requires.setdefault(name, [])
        self.extras_requires[name] = list(
            set(self.extras_requires[name]).union(requirements))

    def removeExtrasRequires(self, name):
        """See interfaces.ISetupBuilder"""
        del self.extras_requires[name]

    def addEntryPoints(self, name, entries):
        """See interfaces.ISetupBuilder"""
        self.entry_points.setdefault(name, [])
        self.entry_points[name] = list(
            set(self.entry_points[name]).union(entries))

    def removeEntryPoints(self, name):
        """See interfaces.ISetupBuilder"""
        del self.entry_points[name]

    def update(self):
        """See interfaces.IBaseBuilder"""
        project = self.getProject()
        if not self.url:
            self.url = u'http://pypi.python.org/pypi/%s' % project.name
        # Update the namespace packages.
        if not self.namespace_packages:
            pieces = project.name.split('.')[:-1]
            pieces.reverse()
            ns = []
            while pieces:
                ns.append(pieces.pop())
                self.namespace_packages.append('.'.join(ns))

    def render(self):
        """See interfaces.IContentBuilder"""
        template = open(base.getTemplatePath('setup.py'), 'r').read()
        project = self.getProject()
        data = dict(
            commentHeader=project.commentHeader.strip(),
            name=project.name,
            version=self.version,
            license=self.license,
            author=self.author,
            author_email=self.author_email,
            description=self.description,
            url=self.url,
            keywords=' '.join(self.keywords),
            namespacePackages='',
            )
        if self.namespace_packages:
            data['namespacePackages'] = ','.join(
                map(repr, self.namespace_packages))

        requirementsStr = ''
        for requirement in self.install_requires:
            requirementsStr += '\n        %r,' % requirement
        data['install_requires'] = requirementsStr

        data['extras_require'] = pprint.pformat(
            self.extras_requires, width=1)

        data['entry_points'] = pprint.pformat(
            self.entry_points, width=1)

        data['classifiers'] = pprint.pformat(
            self.classifiers, width=1)

        return template % data

    def __repr__(self):
        return '<%s for %r>' %(self.__class__.__name__, self.getProject().name)



%(commentHeader)s
"""Setup"""
from setuptools import setup, find_packages

setup (
    name = '%(name)s',
    version = '%(version)s',
    author = u"%(author)s",
    author_email = u"%(author_email)s",
    description = u"%(description)s",
    license = "%(license)s",
    keywords = u"%(keywords)s",
    url = "%(url)s",
    classifiers = %(classifiers)s,
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = [%(namespacePackages)s],
    extras_require = %(extras_require)s,
    install_requires = [
        'setuptools',%(install_requires)s
        ],
    zip_safe = False,
    entry_points = %(entry_points)s,
    )



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Representation of a Python Project

$Id: project.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import shutil
import zope.interface
from rwproperty import getproperty
from zope.schema.fieldproperty import FieldProperty

from z3c.builder.core import base, buildout, interfaces, python, setup


class ProjectBuilder(base.DirectoryBuilder):
    zope.interface.implements(interfaces.IProjectBuilder)

    projectName = FieldProperty(interfaces.IProjectBuilder['projectName'])
    commentHeader = FieldProperty(interfaces.IProjectBuilder['commentHeader'])

    def __init__(self, name, projectName=None):
        super(ProjectBuilder, self).__init__(name)
        if projectName is None:
            projectName = str(name)
        self.projectName = self.dirname = projectName
        self.commentHeader = unicode(open(
            base.getTemplatePath('gpl3-header.py'), 'r').read())


    def update(self):
        self.directoryPath = unicode(self.name)
        self.commentHeader = self.commentHeader %self.__dict__
        for builder in self.values():
            builder.update()

    def write(self, target, force=False):
        """See interfaces.IProjectBuilder"""
        dirPath = os.path.join(target, self.dirname)
        if os.path.exists(dirPath):
            if not force:
                raise interfaces.FileExistsException(dirPath)
            shutil.rmtree(dirPath)
        super(ProjectBuilder, self).write(target)


class BuildoutProjectBuilder(ProjectBuilder):
    zope.interface.implements(interfaces.IBuildoutProjectBuilder)

    @getproperty
    def setup(self):
        return self['setup.py']

    @getproperty
    def buildout(self):
        return self['buildout.cfg']

    @getproperty
    def package(self):
        return self['src'].package

    def __init__(self, name, projectName=None):
        super(BuildoutProjectBuilder, self).__init__(name, projectName)
        # Create the minimum amount files necessary to build a valid project
        self['bootstrap.py'] = base.SimpleFileBuilder(
            u'bootstrap.py', base.getTemplatePath('bootstrap.py'))
        self['setup.py'] = setup.SetupBuilder()
        self['buildout.cfg'] = buildout.BuildoutConfigBuilder()
        self['src'] = SrcDirectoryBuilder()
        self['src'].updateModules(self.name)


class SrcDirectoryBuilder(base.DirectoryBuilder):

    zope.interface.implements(interfaces.IPythonPathRoot)

    def __init__(self):
        super(SrcDirectoryBuilder, self).__init__(u'src')
        self.package = python.PackageBuilder(u'')

    def updateModules(self, projectName):
        modules = list(reversed(unicode(projectName).split('.')))
        builder = self
        while len(modules) > 0:
            name = modules.pop()
            nextBuilder = python.PackageBuilder(name)
            if len(modules) == 0:
                nextBuilder = self.package
                nextBuilder.dirname = nextBuilder.packageName = str(name)
                nextBuilder.name = name
            builder.add(nextBuilder)
            builder = nextBuilder

    def update(self):
        for builder in self.values():
            builder.update()

    def __repr__(self):
        return '<%s for %r>' %(self.__class__.__name__, self.getProject().name)



class %(className)s(AddForm):
    """Add form for %(interface)s"""

    label = u'%(label)s'
    fields = Fields(%(interface)s).select(%(fields)s)

    def create(self, data):
        object = %(factory)s()
        for name, value in data.items():
            setattr(object, name, value)
        return object

    def add(self, object):
        count = 0
        while '%(factory)s-%%i' %%count in self.context:
            count += 1;
        self._name = '%(factory)s-%%i' %%count
        self.context[self._name] = object
        return object

    def nextURL(self):
        return absoluteURL(
            self.context[self._name], self.request) + '/%(next)s'



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Representation of ZCML Configuration files.

$Id: zcml.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import zope.interface
from zope.schema.fieldproperty import FieldProperty
from z3c.builder.core import base, interfaces

ZOPE_NS = 'http://namespaces.zope.org/zope'
BROWSER_NS = 'http://namespaces.zope.org/browser'
ZCML_NS = 'http://namespaces.zope.org/zcml'
Z3C_NS = 'http://namespaces.zope.org/z3c'


class ZCMLDirectiveBuilder(base.BuilderContainer, base.ContentBuilder):
    zope.interface.implements(interfaces.IZCMLDirectiveBuilder)

    namespace = FieldProperty(interfaces.IZCMLDirectiveBuilder['namespace'])
    #name = FieldProperty(interfaces.IZCMLDirectiveBuilder['name'])
    attributes = FieldProperty(interfaces.IZCMLDirectiveBuilder['attributes'])
    indent = FieldProperty(interfaces.IZCMLDirectiveBuilder['indent'])

    _indentSpaces = 2
    _indentAttributeSpaces = 4

    def __init__(self, namespace, name, attributes=None, indent=0):
        super(ZCMLDirectiveBuilder, self).__init__(unicode(name))
        self.namespace = namespace
        if not attributes:
            attributes = {}
        self.attributes = attributes
        self.indent = indent

    def getZCMLBuilder(self):
        """See interfaces.IZCMLDirectiveBuilder"""
        builder = self
        while not interfaces.IZCMLFileBuilder.providedBy(builder):
            builder = builder.__parent__
        return builder

    def add(self, builder):
        """See interfaces.IBuilderContainer"""
        name = base.getUUID()
        self[name] = builder
        return name

    def update(self):
        """See interfaces.IBaseBuilder"""
        zcml = self.getZCMLBuilder()
        if self.namespace is not None and self.namespace not in zcml.namespaces:
            zcml.namespaces.append(self.namespace)
        for builder in self.values():
            builder.indent = self.indent + 1
            builder.update()

    def render(self):
        """See interfaces.IContentBuilder"""
        output = ''
        prefix = ''
        if self.namespace:
            prefix = self.namespace.rsplit('/')[-1] + ':'
        # Write XML node
        indent = ' ' * self._indentSpaces * self.indent
        attrIndent = indent + ' '*self._indentAttributeSpaces
        output += indent + '<%s%s' %(prefix, self.name)
        if self.attributes:
            for name, attr in reversed(self.attributes.items()):
                output += '\n' + attrIndent + '%s="%s"' %(name, attr)
            output += '\n' + attrIndent
        if not len(self):
            output += '/>\n'
            return output
        output += '>\n'
        # Write all sub-builders.
        for builder in self.values():
            output += builder.render()
            output += '\n'
        # Close XML node.
        output += indent + '</%s%s>\n' %(prefix, self.name)
        # Return result.
        return output


class ZCMLFileBuilder(ZCMLDirectiveBuilder, base.FileBuilder):
    zope.interface.implements(interfaces.IZCMLFileBuilder)

    i18n_domain = FieldProperty(interfaces.IZCMLFileBuilder['i18n_domain'])
    namespaces = FieldProperty(interfaces.IZCMLFileBuilder['namespaces'])

    def __init__(self, name):
        ZCMLDirectiveBuilder.__init__(self, None, 'configure')
        base.FileBuilder.__init__(self, self.name, str(name))

    def update(self):
        """See interfaces.IBaseBuilder"""
        self.namespaces = []
        if not self.i18n_domain:
            project = self.getProject()
            self.i18n_domain = project.name
        super(ZCMLFileBuilder, self).update()
        # Create list of attributes
        self.attributes = {}
        if self.i18n_domain:
            self.attributes['i18n_domain'] = self.i18n_domain
        for namespace in self.namespaces:
            name = namespace.rsplit('/', 1)[-1]
            self.attributes['xmlns:'+name] = namespace

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.filename)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Buildout Configuration Builder

$Id: buildout.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import zope.interface
from zope.schema.fieldproperty import FieldProperty
from z3c.builder.core import base, interfaces

class PartBuilder(base.ContentBuilder):
    zope.interface.implements(interfaces.IPartBuilder)

    values = FieldProperty(interfaces.IPartBuilder['values'])
    autoBuild = FieldProperty(interfaces.IPartBuilder['autoBuild'])

    def __init__(self, name, values=None, autoBuild=True):
        super(PartBuilder, self).__init__(name)
        if values is None:
            values = []
        self.values = list(values)
        self.autoBuild = autoBuild

    def addValue(self, key, value):
        self.values.append((key, value))

    def removeValue(self, key):
        for item in self.values:
            if item[0] == key:
                self.values.remove(item)
                return

    def render(self):
        output = ''
        output += '[%s]\n' %self.name
        for key, value in self.values:
            output += '%s = %s\n' %(key, value)
        return output


class BuildoutConfigBuilder(base.BuilderContainer, base.FileBuilder):
    zope.interface.implements(interfaces.IBuildoutConfigBuilder)

    extends = FieldProperty(interfaces.IBuildoutConfigBuilder['extends'])
    names = FieldProperty(interfaces.IBuildoutConfigBuilder['names'])

    def __init__(self):
        super(BuildoutConfigBuilder, self).__init__(u'buildout.cfg')
        self.extends = [u'http://download.zope.org/zope3.4/3.4.0/versions.cfg']
        self.names = []

    def update(self):
        self.names = [part.name for part in self.values()
                      if part.autoBuild]
        for part in self.values():
            part.update()

    def render(self):
        output = ''
        output += '[buildout]\n'
        output += 'extends = %s\n' %('   \n'.join(self.extends))
        output += 'develop = .\n'
        output += 'parts = %s\n' %' '.join(self.names)
        output += 'versions = versions\n'
        if len(self):
            output += '\n'
        for part in self.values():
            output += part.render()
            output += '\n'
        return output

    def __repr__(self):
        return '<%s for %r>' %(self.__class__.__name__, self.getProject().name)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Python Builder Components

$Id: python.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import types
import zope.interface
from zope.schema.fieldproperty import FieldProperty
from z3c.builder.core import base, interfaces


class ModuleBuilderGetter(object):
    zope.interface.implements(interfaces.IModuleBuilderGetter)

    def getModuleBuilder(self):
        module = self
        while not interfaces.IModuleBuilder.providedBy(module):
            module = module.__parent__
            if module is None:
                raise ValueError(
                    'No module builder was found and the root node of '
                    'the project tree was reached.')
        return module


class PackageBuilder(base.DirectoryBuilder):
    zope.interface.implements(interfaces.IPackageBuilder)

    initTemplate = base.getTemplatePath('__init__.py')
    initNamespaceTemplate = base.getTemplatePath('__init__namespace.py')
    packageName = FieldProperty(interfaces.IPackageBuilder['packageName'])

    def __init__(self, name, packageName=None):
        if packageName is None:
            packageName = str(name)
        super(PackageBuilder, self).__init__(name, packageName)
        self.packageName = packageName

    def getPythonPath(self):
        if interfaces.IPythonPathRoot.providedBy(self.__parent__):
            return self.packageName
        if self.__parent__ is None:
            return self.packageName
        return self.__parent__.getPythonPath() + '.' + str(self.packageName)

    def update(self):
        """See interfaces.IBaseBuilder"""
        super(PackageBuilder, self).update()
        matches = [subBuilder for subBuilder in self.values()
                   if (interfaces.IFileBuilder.providedBy(subBuilder) and
                       subBuilder.filename == '__init__.py')]
        if matches:
            return
        template = self.initTemplate
        try:
            project = self.getProject()
        except ValueError:
            project = None
        if project and self.getPythonPath() in project.setup.namespace_packages:
            template = self.initNamespaceTemplate
        self['__init__.py'] = base.SimpleFileBuilder(
            u'__init__.py', template)


class ModuleBuilder(base.BuilderContainer, base.FileBuilder):
    zope.interface.implements(interfaces.IModuleBuilder)

    moduleName = FieldProperty(interfaces.IModuleBuilder['moduleName'])
    imports = FieldProperty(interfaces.IModuleBuilder['imports'])

    def __init__(self, name, moduleName=None):
        super(ModuleBuilder, self).__init__(name)
        if moduleName is None:
            moduleName = str(name)[:-3]
        self.moduleName = moduleName
        self.filename = str(name)

    def getPythonPath(self):
        return self.__parent__.getPythonPath() + '.' + str(self.moduleName)

    def update(self):
        self.imports = []
        super(ModuleBuilder, self).update()

    def render(self):
        """See interfaces.IContentBuilder"""
        output = []
        # Render comment header.
        project = self.getProject()
        output.append(project.commentHeader)
        # Render module doc string.
        output.append('"""Module Documentation"""\n')
        # Render imports making sure none is duplicated.
        for path in sorted(set(self.imports)):
            module, name = tuple(path.rsplit('.', 1))
            if module != self.getPythonPath():
                output.append('from %s import %s\n' % (module, name))
        output.append('\n\n')
        # Render all sub-builders.
        for subBuilder in self.values():
            output.append(subBuilder.render())
            output.append('\n\n')
        # Return a concatenated version of all pieces.
        return ''.join(output)


class FieldBuilder(ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IFieldBuilder)

    fieldName = FieldProperty(interfaces.IFieldBuilder['fieldName'])
    type = FieldProperty(interfaces.IFieldBuilder['type'])
    attributes = FieldProperty(interfaces.IFieldBuilder['attributes'])
    indent = FieldProperty(interfaces.IFieldBuilder['indent'])

    _numberOfSpaces = 4

    def __init__(self, name, type, indent=0, **attributes):
        self.name = name
        self.type = type
        self.indent = indent
        self.attributes = attributes

    def update(self):
        self.getModuleBuilder().imports.append(self.type)

    def render(self):
        indent = ' ' * (self.indent * self._numberOfSpaces)
        output = ''
        output += indent + '%s = %s(' %(self.name, self.type.rsplit('.')[-1])
        if self.attributes:
            output += '\n'
        indent += ' ' * (self._numberOfSpaces)
        for item in reversed(self.attributes.items()):
            output += indent + '%s=%r,\n' %item
        if self.attributes:
            output += indent
        output += ')\n'
        return output


class FunctionBuilder(ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IFunctionBuilder)

    fieldName = FieldProperty(interfaces.IFieldBuilder['fieldName'])
    type = FieldProperty(interfaces.IFieldBuilder['type'])
    attributes = FieldProperty(interfaces.IFieldBuilder['attributes'])
    indent = FieldProperty(interfaces.IFieldBuilder['indent'])

    _numberOfSpaces = 4

    def __init__(self, name, args=(), kwargs=None,
                 docstring='', indent=0, code=''):
        self.name = name
        self.args = args
        self.kwargs = (kwargs and dict(kwargs)) or {}
        self.docstring = docstring
        self.indent = indent
        self.code = code

    def render(self):
        # Generate the proper indentation level.
        indent = ' ' * (self.indent * self._numberOfSpaces)
        output = ''
        # Render the first part of the function header.
        output += indent+'def %s(' %str(self.name)
        # Render the position arguments.
        output += ', '.join([str(arg) for arg in self.args])
        if self.args and self.kwargs:
            output += ', '
        # Render keyword arguments.
        output += ', '.join(['%s=%r' %item for item in self.kwargs.items()])
        output += '):\n'
        # Render body of function
        indent += self._numberOfSpaces * ' '
        if self.docstring:
            output += indent + '"""%s"""' %self.docstring
        if self.code:
            output += '\n'
            for line in self.code.split('\n'):
                output += indent + line + '\n'
        if not self.code and not self.docstring:
            output += indent + 'pass'
        output += '\n'
        return output


class InterfaceBuilder(ModuleBuilderGetter,
                       base.BuilderContainer, base.ContentBuilder):
    zope.interface.implements(interfaces.IInterfaceBuilder)

    docstring = FieldProperty(interfaces.IInterfaceBuilder['docstring'])
    bases = FieldProperty(interfaces.IInterfaceBuilder['bases'])

    def __init__(self, name):
        super(InterfaceBuilder, self).__init__(name)
        self.docstring = ''
        self.bases = []

    def add(self, builder):
        if isinstance(builder, (FunctionBuilder, FieldBuilder)):
            builder.indent = 1
        return super(InterfaceBuilder, self).add(builder)

    def getPythonPath(self):
        return self.__parent__.getPythonPath() + '.' + str(self.name)

    def update(self):
        if len(self.bases) == 0:
            self.bases = ['zope.interface.Interface']
        self.getModuleBuilder().imports += self.bases
        for builder in self.values():
            builder.update()

    def render(self):
        bases = [path.rsplit('.', 1)[-1] for path in self.bases]
        output = ''
        output += 'class %s(%s):\n' %(str(self.name), ', '.join(bases))
        output += '    """%s"""\n' %self.docstring
        for builder in self.values():
            output += builder.render()
            output += '\n'
        return output


class ClassFromInterfaceBuilder(ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IClassFromInterfaceBuilder)

    interface = FieldProperty(
        interfaces.IClassFromInterfaceBuilder['interface'])
    bases = FieldProperty(
        interfaces.IClassFromInterfaceBuilder['bases'])
    docstring = FieldProperty(
        interfaces.IClassFromInterfaceBuilder['docstring'])
    implementations = FieldProperty(
        interfaces.IClassFromInterfaceBuilder['implementations'])

    def __init__(self, name, interface, bases=None):
        super(ClassFromInterfaceBuilder, self).__init__(name)
        self.interface = interface
        self.bases = list(bases) if bases is not None else []
        self.docstring = ''
        self.implementations = {}

    def _resolve(self, path):
        pieces = path.split('.')
        # Find package root.
        root = self
        while not interfaces.IPythonPathRoot.providedBy(root.__parent__):
            root = root.__parent__
        # Find builder
        path = ''
        builder = root
        for piece in pieces:
            if path:
                path += '.'
            path += piece
            for sub in builder.values():
                if (hasattr(sub, 'getPythonPath') and
                    path == sub.getPythonPath()):
                    builder = sub
                    break
        return builder

    def addImplementation(self, name, code):
        self.implementations[name] = code

    def getPythonPath(self):
        return self.__parent__.getPythonPath() + '.' + str(self.name)

    def update(self):
        # Determine the bases
        if len(self.bases) == 0:
            self.bases = ['object']
        self.getModuleBuilder().imports += self.bases
        # Compute the interface, if necessary
        if isinstance(self.interface, types.StringTypes):
            self.interface = self._resolve(self.interface)
        # Add import for interface
        self.__parent__.imports.append('zope.interface.implements')
        self.__parent__.imports.append(self.interface.getPythonPath())
        # Generate docstring if necessary
        if not self.docstring:
            self.docstring = "Implementation of ``%s``" % self.interface.getPythonPath()
        # Generate fields and methods
        self.fields = [entry for entry in self.interface.values()
                       if isinstance(entry, FieldBuilder)]
        self.methods = [entry for entry in self.interface.values()
                       if isinstance(entry, FunctionBuilder)]
        # Add imports if needed
        if len(self.fields):
            self.__parent__.imports.append(
                'zope.schema.fieldproperty.FieldProperty')

    def render(self):
        bases = [path.rsplit('.', 1)[-1] for path in self.bases]
        output = ''
        output += 'class %s(%s):\n' %(self.name, ', '.join(bases))
        output += '    """%s"""\n' %self.docstring
        output += '    implements(%s)\n\n' %self.interface.name
        for field in self.fields:
            output += "    %s = FieldProperty(%s['%s'])\n" %(
                field.name, self.interface.name, field.name)
        output += '\n'
        for method in self.methods:
            output += '    def %s(' %method.name
            allargs = ['self']
            allargs += [str(arg) for arg in method.args]
            allargs += ['%s=%r' %item for item in method.kwargs.items()]
            output += ', '.join(allargs)
            output += '):\n'
            output += '        """See ``%s``."""\n' %self.interface.getPythonPath()
            if method.name in self.implementations:
                impl = self.implementations[method.name]
                output += ' '*8
                output += impl.replace('\n', '\n'+' '*8)
            output += '\n'
        return output



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Setup.

$Id: test_doc.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import re
import shutil
import sys
import unittest
import tempfile
import logging
import subprocess
import tempfile
import zope.component
from zope.configuration import xmlconfig
from zope.testing import renormalizing, doctest

from z3c.builder.core import base, testing

def fullSetUp(test):
    testing.buildSetUp(test)
    testing.loggerSetUp(test)

def fullTearDown(test):
    testing.buildTearDown(test)
    testing.loggerTearDown(test)

def test_suite():
    checker = renormalizing.RENormalizing([
       (re.compile(
            '[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}.[0-9]{6}'),
            '<DATETIME>'),
       (re.compile(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'),
            '<UUID>'),
       ])

    return unittest.TestSuite((

        doctest.DocFileSuite(
            '../README.txt',
            setUp=fullSetUp, tearDown=fullTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../project.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../python.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../setup.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../buildout.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../zcml.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../form.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            checker=checker),

        doctest.DocFileSuite(
            '../example.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS|doctest.REPORT_NDIFF,
            checker=checker),

        ))



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Representation of Form view classes.

$Id: form.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import zope.interface
from zope.schema.fieldproperty import FieldProperty
from z3c.builder.core import base, interfaces, python

class AddFormBuilder(python.ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IAddFormBuilder)

    interface = FieldProperty(interfaces.IAddFormBuilder['interface'])
    factory = FieldProperty(interfaces.IAddFormBuilder['factory'])
    fields = FieldProperty(interfaces.IAddFormBuilder['fields'])
    next = FieldProperty(interfaces.IAddFormBuilder['next'])
    label = FieldProperty(interfaces.IAddFormBuilder['label'])

    def __init__(self, name, interface, factory,
                 fields=(), next='index.html', label=u'Add Form'):
        super(AddFormBuilder, self).__init__(name)
        self.interface = interface
        self.factory = factory
        self.fields = tuple([str(f) for f in fields])
        self.next = next
        self.label = label

    def update(self):
        """See interfaces.IBaseBuilder"""
        self.getModuleBuilder().imports += [
            'z3c.form.form.AddForm',
            'z3c.form.field.Fields',
            'zope.traversing.browser.absoluteURL',
            self.interface,
            self.factory]

    def render(self):
        """See interfaces.IContentBuilder"""
        template = open(base.getTemplatePath('add-form.py'), 'r').read()
        output = (template %{
            'interface': self.interface.rsplit('.', 1)[-1],
            'label': self.label,
            'fields': ', '.join([repr(field) for field in self.fields]),
            'factory': self.factory.rsplit('.', 1)[-1],
            'next': self.next,
            'className':self.name,
            })
        output += '\n'
        return output


class EditFormBuilder(python.ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IEditFormBuilder)

    interface = FieldProperty(interfaces.IEditFormBuilder['interface'])
    fields = FieldProperty(interfaces.IEditFormBuilder['fields'])
    label = FieldProperty(interfaces.IEditFormBuilder['label'])

    def __init__(self, name, interface, fields=(), label=u'Edit Form'):
        super(EditFormBuilder, self).__init__(name)
        self.interface = interface
        self.fields = tuple([str(f) for f in fields])
        self.label = label

    def update(self):
        self.getModuleBuilder().imports += [
            'z3c.form.form.EditForm',
            'z3c.form.field.Fields',
            self.interface]

    def render(self):
        template = open(base.getTemplatePath('edit-form.py'), 'r').read()
        output = (template %{
            'interface': self.interface.rsplit('.', 1)[-1],
            'label': self.label,
            'fields': ', '.join([repr(field) for field in self.fields]),
            'className':self.name,
            })
        output += '\n'
        return output


class SimpleDisplayFormBuilder(python.ModuleBuilderGetter, base.ContentBuilder):
    zope.interface.implements(interfaces.IDisplayFormBuilder)

    interface = FieldProperty(interfaces.IDisplayFormBuilder['interface'])
    fields = FieldProperty(interfaces.IDisplayFormBuilder['fields'])
    template = FieldProperty(interfaces.IDisplayFormBuilder['template'])

    def __init__(self, name, interface, template=None, fields=()):
        super(SimpleDisplayFormBuilder, self).__init__(name)
        self.interface = interface
        self.template = template
        self.fields = tuple([str(f) for f in fields])

    def update(self):
        self.getModuleBuilder().imports += [
            'z3c.form.form.Form',
            'z3c.form.form.DisplayForm',
            'z3c.form.field.Fields',
            self.interface]
        # Add the template for the form.
        if self.template:
            browserPackage = self.getModuleBuilder().__parent__
            if 'display.pt' not in browserPackage:
                zpt = base.SimpleFileBuilder(
                    u'display.pt', base.getTemplatePath('simple-display-form.pt'))
                browserPackage.add(zpt)
                zpt.update()

    def render(self):
        template = open(
            base.getTemplatePath('simple-display-form.py'), 'r').read()
        output = (template %{
            'interface': self.interface.rsplit('.', 1)[-1],
            'fields': ', '.join([repr(field) for field in self.fields]),
            'template': self.template,
            'className': self.name,
            })
        output += '\n'
        return output



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup

$Id: setup.py 98391 2009-03-27 08:50:42Z pcardune $
"""
import os
import xml.sax.saxutils
from setuptools import setup, find_packages

def read(*rnames):
    text = open(os.path.join(os.path.dirname(__file__), *rnames)).read()
    text = unicode(text, 'utf-8').encode('ascii', 'xmlcharrefreplace')
    return xml.sax.saxutils.escape(text)

setup (
    name='z3c.builder.core',
    version='0.1.0',
    author = "Paul Carduner, Stephan Richter, and hopefully others...",
    author_email = "zope-dev@zope.org",
    description = "A utility to help jump start Zope 3 projects",
    long_description=(
        read('README.txt')
        +"\n\n"+
        read('CHANGES.txt')
        ),
    license = "ZPL 2.1",
    keywords = "zope3 project builder",
    url = 'http://pypi.python.org/pypi/z3c.builder.core',
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['z3c'],
    extras_require = dict(
        test = [
            'zope.testing',
            'z3c.coverage',
            ],
        docs = ['Sphinx'],
        ),
    install_requires = [
        'rwproperty',
        'setuptools',
        'zc.buildout',
        'zope.component',
        'zope.configuration',
        'zope.container',
        'zope.interface',
        'zope.schema',
        'lxml',
        ],
    zip_safe = False,
    )
